# -*- coding: UTF-8 -*-
# @Time    : 2020-06-01 09:00
# @File    : __init__.py.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function

import atexit
import sys
from . import mlogger
import logging
__exit_flag = False


def handle_exit():
  global __exit_flag
  if not __exit_flag:
    # project/experiment
    if mlogger.getEnv() is None or mlogger.getEnv().dashboard is None:
      return

    #
    project = mlogger.getEnv().dashboard.project
    experiment = mlogger.getEnv().dashboard.experiment_name

    if project is not None and experiment is not None:
      # 更新未上传数据
      mlogger.update()

      # 日志退出
      logging.info('Prepare exit {}/{} experiment logger'.format(project, experiment))
      mlogger.exit()

      # print log
      logging.info('Finish {}/{} experiment logger'.format(project, experiment))


def handle_exception(exc_type, exc_value, exc_traceback):
  global __exit_flag
  __exit_flag = True
  if mlogger.getEnv() is None or mlogger.getEnv().dashboard is None:
    return

  project = mlogger.getEnv().dashboard.project
  experiment = mlogger.getEnv().dashboard.experiment_name

  if project is not None and experiment is not None:
    # error
    mlogger.error()
    # print error info
    print(exc_traceback)
    # print log
    logging.error('Error {}/{} experiment logger'.format(project, experiment))


sys.excepthook = handle_exception
atexit.register(handle_exit)



# -*- coding: UTF-8 -*-
# @Time    : 2020-06-29 22:30
# @File    : base.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function

import numpy as np
import time

from collections import defaultdict, OrderedDict
from antvis.client import mlogger
import logging
LOG_FORMAT = "%(asctime)s - %(levelname)s - %(filename)s[:%(lineno)d] - %(message)s"
DATE_FORMAT = "%m/%d/%Y %H:%M:%S %p"
logging.basicConfig(level=logging.DEBUG, format=LOG_FORMAT, datefmt=DATE_FORMAT)


class Base(object):
    charts = {}
    
    def __init__(self, plot_title, logger_type='metric', **kwargs):
        """ Basic metric
        """
        self.plot_title = plot_title
        self.chart_x_axis = ''
        self.chart_y_axis = ''
        self.channel = None
        self.chart = None
        self.reset_hooks_on_update()
        self.reset_hooks_on_log()
        self.time = 0
        self.logger_type = logger_type
        self.channel_config = kwargs
        self._val = None

    def config(self, **kwargs):
        if 'x_axis' in kwargs:
            self.chart.x_axis = kwargs['x_axis']
            kwargs.pop('x_axis')

        if 'y_axis' in kwargs:
            self.chart.y_axis = kwargs['y_axis']
            kwargs.pop('y_axis')

        self.channel.params.update(kwargs)

    def update(self, *args, **kwargs):
        self._update(*args, **kwargs)
        for hook in self.hooks_on_update:
            hook()
        self.time += 1
        return self

    @property
    def value(self):
        raise NotImplementedError

    @property
    def name(self):
        return self.channel.channel_name
    
    @name.setter
    def name(self, val):
        self.channel.channel_name = val
        
        group_name = ''
        if '/' in val:
            group_name, _ = val.split('/')
            
        if self.chart is None:
            group_plot_title = self.plot_title
            if group_name != '':
                group_plot_title = '{}/{}'.format(group_name, self.plot_title)

            if group_plot_title not in Base.charts:
                Base.charts[group_plot_title] = mlogger.getEnv().create_chart([], group_plot_title)
            self.chart = Base.charts[group_plot_title]
            self.chart.chart_x_axis = self.chart_x_axis
            self.chart.chart_y_axis = self.chart_y_axis
        
        self.chart.bind_channel(self.channel)
        
    def reset_hooks_on_update(self):
        self.hooks_on_update = ()

    def reset_hooks_on_log(self):
        self.hooks_on_log = ()

    def hook_on_update(self, hook):
        self.hooks_on_update += (hook,)

    def hook_on_log(self, hook):
        self.hooks_on_log += (hook,)

    def _update(self, *args, **kwargs):
        raise NotImplementedError("_update should be re-implemented for each metric")

    def _get(self, data):
        return data

    def get(self, kind='current'):
        assert(kind in ['current', 'history'])
        # 1.step 优先使用本地数据
        if self.value is not None and kind == 'current':
            return {'value': self.value, 'type': 'current'}

        # 2.step 使用远程数据
        channel_name = self.channel.channel_name
        chart_name = self.chart.chart_title

        data = \
            mlogger.getEnv().dashboard.\
                rpc.experiment.chart.get(chart_name=chart_name,
                                         channel_name=channel_name)
        if data['status'] == 'OK':
            return {'value': self._get(data['content']), 'type': 'history'}
        return None


# -*- coding: UTF-8 -*-
# @Time    : 2021/1/3 8:45 下午
# @File    : main3.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
import antvis.client.mlogger as mlogger
import numpy as np
import time


if __name__ == "__main__":
  mlogger.config('59.110.164.237',
                 8080,
                 token='acee26bd077d4f55afa8dad3f5828fb2')
  project = 'you'
  experiment = 'jian10'
  mlogger.activate(project, experiment)

  # 测试实验标签获取
  print(mlogger.tag.learning_rate.get())
  print(mlogger.tag.batch_size.get())
  print(mlogger.tag.weight_decay.get())

  # 获得图表数据
  container_1 = mlogger.Container()
  container_1.growth_curve = mlogger.metric.Simple('growth')
  print(container_1.growth_curve.get())

  # 更新日志文件
  container_1.after = mlogger.FileLogger('after-1')
  container_1.before = mlogger.FileLogger('before-1')

  container_1.after.update('/Users/jian/Downloads/bdd79fec-1ad0-405f-bea9-557300b33993-db928c27-f234-463b-a073-81941824f701_1-200413093331.zip')
  container_1.before.update('/Users/jian/Downloads/character.jpg')


# -*- coding: UTF-8 -*-
# @Time    : 2020-06-25 23:28
# @File    : __init__.py.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
import numpy as np
import time
import uuid

from collections import defaultdict, OrderedDict
from antvis.client.mlogger.metric.base import *
from antvis.client.dashboard import *
from antvis.client import mlogger


class Simple(Base):
    def __init__(self, plot_title, **kwargs):
        super(Simple, self).__init__(plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = ''
        
        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='LINE', **self.channel_config)

    @property
    def value(self):
        return self._val
    
    def _update(self, val):
        self._val = float(val)
        self.channel.update(self.time, self.value)
        
        
class Accumulator_(Base):
    """
    Credits to the authors of pytorch/tnt for this.
    """
    def __init__(self,plot_title=None, **kwargs):
        super(Accumulator_, self).__init__(plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = ''

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='LINE', **self.channel_config)
        self.reset()
        
    def reset(self):
        self._avg = 0
        self._total_weight = 0
        return self

    def _update(self, val, weighting=1):
        val, weighting = float(val), float(weighting)
        assert weighting > 0
        r = self._total_weight / (weighting + self._total_weight)
        self._avg = r * self._avg + (1 - r) * val
        self._total_weight += weighting
        
        # update
        self.channel.update(self.time, self.value)
        
    @property
    def value(self):
        raise NotImplementedError("Accumulator should be subclassed")


class Average(Accumulator_):
    def __init__(self, plot_title=None, **kwargs):
        super(Average, self).__init__(plot_title, **kwargs)

    @property
    def value(self):
        return self._avg


class Sum(Accumulator_):
    def __init__(self, plot_title=None, **kwargs):
        super(Sum, self).__init__(plot_title, **kwargs)

    @property
    def value(self):
        return self._avg * self._total_weight


class Maximum(Base):
    def __init__(self, plot_title=None, **kwargs):
        super(Maximum, self).__init__(plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = ''

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='LINE', **self.channel_config)
        self.reset()

    def reset(self):
        self._val = -np.inf
        self.hooks_on_new_max = ()
        return self

    def _update(self, val, n=None):
        val = float(val)
        if val > self._val:
            self._val = val
            for hook in self.hooks_on_new_max:
                hook()
                
        self.channel.update(self.time, self.value)
        
    def hook_on_new_max(self, hook):
        self.hooks_on_new_max += (hook,)

    @property
    def value(self):
        return self._val


class Minimum(Base):
    def __init__(self, plot_title=None, **kwargs):
        super(Minimum, self).__init__(plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = ''

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='LINE', **self.channel_config)
        self.reset()

    def reset(self):
        self._val = np.inf
        self.hooks_on_new_min = ()
        return self

    def _update(self, val, n=None):
        val = float(val)
        if val < self._val:
            self._val = val
            for hook in self.hooks_on_new_min:
                hook()

        self.channel.update(self.time, self.value)

    def hook_on_new_min(self, hook):
        self.hooks_on_new_min += (hook,)

    @property
    def value(self):
        return self._val





# -*- coding: UTF-8 -*-
# @Time    : 2020/8/13 7:50 下午
# @File    : __init__.py.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from torch import nn
from antvis.client.mlogger.metric.base import *
import torch
import numpy as np
import uuid


class Image(nn.Module, Base):
    def __init__(self, plot_title, **kwargs):
        nn.Module.__init__(self)
        Base.__init__(self, plot_title, 'complex', **kwargs)
        self.channel =\
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='IMAGE',
                                            **self.channel_config)

    def forward(self, x):
        # tensor NCHW
        if len(x.shape) == 4 and x.shape[1] == 3 and x.dtype == torch.uint8:
            data = x.cpu().data.numpy()
            for index in range(data.shape[0]):
                self.update(data[index].transpose((1,2,0)))
        else:
            print('no image data, couldnt log')

        return x

    @property
    def value(self):
        return self._val

    def _update(self, val):
        if type(val) == list:
            val = np.array(val)

        if type(val) != np.ndarray:
            print('image logger dont support non numpy.ndarray data')
            raise NotImplementedError

        assert (len(val.shape) == 2 or len(val.shape) == 3)
        if len(val.shape) == 3:
            assert (val.shape[2] == 3)

        self._val = val
        self.channel.update(self.time, self.value)


class Histogram(nn.Module, Base):
    def __init__(self, plot_title, **kwargs):
        nn.Module.__init__(self)
        Base.__init__(self, plot_title, 'complex', **kwargs)
        self.channel = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='HISTOGRAM',
                                            **self.channel_config)
        self.multi_channels = [self.channel]

    def forward(self, x):
        # NCHW
        if x is not None:
            data = x.cpu().data.numpy()
            self.update(data)
        return x

    @property
    def value(self):
        return self._val

    def _update(self, val):
        if len(val.shape) == 1:
            # 单通道数据
            self.multi_channels[0].update(self.time, val)
        else:
            # 多通道数据
            if val.shape[0] > len(self.multi_channels):
                # 将新增通道加入图表
                channel_num = len(self.multi_channels)
                for index in range(val.shape[0] - channel_num):
                    new_channel = \
                        mlogger.getEnv().create_channel('%s-%d'%(self.channel.channel_name, index+channel_num),
                                                        channel_type='HISTOGRAM',
                                                        **self.channel_config)
                    self.chart.bind_channel(new_channel)
                    self.multi_channels.append(new_channel)

            for index in range(val.shape[0]):
                self.multi_channels[index].update(self.time, val[index])

        self._val = val


class Simple(nn.Module, Base):
    def __init__(self, plot_title, **kwargs):
        nn.Module.__init__(self)
        Base.__init__(self, plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = ''

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='LINE', **self.channel_config)

    def forward(self, x):
        self.update(x.item())
        return x

    @property
    def value(self):
        return self._val

    def _update(self, val):
        self._val = float(val)
        self.channel.update(self.time, self.value)


class Accumulator_(nn.Module, Base):
    """
    Credits to the authors of pytorch/tnt for this.
    """

    def __init__(self, plot_title, **kwargs):
        nn.Module.__init__(self)
        Base.__init__(self, plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = ''

        self.channel = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.reset()

    def reset(self):
        self._avg = None
        self._total_weight = 0
        return self

    def _update(self, val, weighting=1):
        if self._avg is None:
            self._avg = 0

        val, weighting = float(val), float(weighting)
        assert weighting > 0
        r = self._total_weight / (weighting + self._total_weight)
        self._avg = r * self._avg + (1 - r) * val
        self._total_weight += weighting

        # update
        self.channel.update(self.time, self.value)

    @property
    def value(self):
        raise NotImplementedError("Accumulator should be subclassed")

    def forward(self, x):
        self.update(x.item())
        return x


class Average(Accumulator_):
    def __init__(self, plot_title, **kwargs):
        Accumulator_.__init__(self, plot_title, **kwargs)

    @property
    def value(self):
        return self._avg


class Sum(Accumulator_):
    def __init__(self, plot_title, **kwargs):
        Accumulator_.__init__(self, plot_title, **kwargs)

    @property
    def value(self):
        if self._avg is None:
            return None

        return self._avg * self._total_weight


class Maximum(nn.Module, Base):
    def __init__(self, plot_title, **kwargs):
        nn.Module.__init__(self)
        Base.__init__(self, plot_title, **kwargs)

        self.chart_x_axis = 'time'
        self.chart_y_axis = ''

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='LINE', **self.channel_config)
        self.reset()

    def reset(self):
        self._val = None
        self.hooks_on_new_max = ()
        return self

    def _update(self, val, n=None):
        if self._val is None:
            self._val = -np.inf

        val = float(val)
        if val > self._val:
            self._val = val
            for hook in self.hooks_on_new_max:
                hook()

        self.channel.update(self.time, self.value)

    def hook_on_new_max(self, hook):
        self.hooks_on_new_max += (hook,)

    @property
    def value(self):
        return self._val

    def forward(self, x):
        self.update(x.item())
        return x


class Minimum(nn.Module, Base):
    def __init__(self, plot_title, **kwargs):
        nn.Module.__init__(self)
        Base.__init__(self, plot_title, **kwargs)

        self.chart_x_axis = 'time'
        self.chart_y_axis = ''

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='LINE', **self.channel_config)
        self.reset()

    def reset(self):
        self._val = None
        self.hooks_on_new_min = ()
        return self

    def _update(self, val, n=None):
        if self._val is None:
            self._val = np.inf

        val = float(val)
        if val < self._val:
            self._val = val
            for hook in self.hooks_on_new_min:
                hook()

        self.channel.update(self.time, self.value)

    def hook_on_new_min(self, hook):
        self.hooks_on_new_min += (hook,)

    @property
    def value(self):
        return self._val

    def forward(self, x):
        self.update(x.item())
        return x


# -*- coding: UTF-8 -*-
# @Time    : 2020-06-01 10:32
# @File    : dashboard.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from antvis.client.job import *
from antvis.utils.utils import *
from antvis.client.httprpc import *
import json
import requests
import logging


class Dashboard(object):
    def __init__(self,
                 token=None,
                 experiment_uuid=None,
                 **kwargs):
        self._dashboard_ip = '121.40.35.60'         # ip
        self._dashboard_port = 8000                 # port
        self._dashboard_prefix = 'api/antvis'
        self.launch_time = timestamp()
        self.quiet = False
        self._project = None
        self._experiment_name = None
        
        self._server = kwargs.get('server', "EXPLORE")
        self._experiment_uuid = None
        self._token = None
        self._hyper_parameter = ''
        self._stage = ''       # TRAIN, CHALLENGE, EXPERIMENT, ALL
        
        # 信息提交模块
        self.job = Job(self)
        self.job.start()

        # http rpc
        self.rpc = HttpRpc("v1",
                           self.dashboard_prefix,
                           self.dashboard_ip,
                           self.dashboard_port)
        if token is not None:
            self.token = token                          # 任务token
        if experiment_uuid is not None:
            self.experiment_uuid = experiment_uuid      # 实验uuid
        
    @property
    def project(self):
        return self._project
    
    @project.setter
    def project(self, val):
        self._project = val
    
    @property
    def experiment_name(self):
        return self._experiment_name
    
    @experiment_name.setter
    def experiment_name(self, val):
        self._experiment_name = val
    
    @property
    def server(self):
        return self._server
    
    @server.setter
    def server(self, val):
        self._server = val
        if self.rpc is not None:
            self.rpc.data.update({'experiment_context': self.server})

    @property
    def experiment_uuid(self):
        return self._experiment_uuid
    
    @experiment_uuid.setter
    def experiment_uuid(self, val):
        self._experiment_uuid = val
        if self.rpc is not None:
            self.rpc.data.update({'experiment_uuid': self.experiment_uuid})
    
    @property
    def token(self):
        return self._token
    
    @token.setter
    def token(self, val):
        self._token = val
        if self.rpc is not None:
            self.rpc.token = val
    
    @property
    def experiment_hyper_parameter(self):
        return self._hyper_parameter
    
    @experiment_hyper_parameter.setter
    def experiment_hyper_parameter(self, val):
        self._hyper_parameter = val
    
    @property
    def dashboard_ip(self):
        return self._dashboard_ip
    
    @dashboard_ip.setter
    def dashboard_ip(self, val):
        self._dashboard_ip = val
        if self.rpc is not None:
            self.rpc.ip = val
    
    @property
    def dashboard_port(self):
        return self._dashboard_port

    @dashboard_port.setter
    def dashboard_port(self, val):
        self._dashboard_port = val
        if self.rpc is not None:
            self.rpc.port = int(val)
    
    @property
    def dashboard_prefix(self):
        return self._dashboard_prefix
    
    @dashboard_prefix.setter
    def dashboard_prefix(self, val):
        self._dashboard_prefix = val
        if self.rpc is not None:
            self.rpc.prefix = val
    
    @property
    def experiment_stage(self):
        return self._stage
    
    @experiment_stage.setter
    def experiment_stage(self, val):
        self._stage = val

    def create_channel(self, channel_name, channel_type, **kwargs):
        return self.job.create_channel(channel_name, channel_type, **kwargs)

    def create_chart(self, chart_channels, chart_title, chart_x_axis="x", chart_y_axis="y", **kwargs):
        return self.job.create_chart(chart_channels, chart_title, chart_x_axis, chart_y_axis, **kwargs)
    
    def create_project(self, *args, **kwargs):
        # 1.step 配置dashboard
        self.configure(*args, **kwargs)
        
        # 2.step 创建项目(获得 experiment_uuid)
        response = \
            self.rpc.experiment.post(task_name=self.project,
                                     experiment_name=self.experiment_name)
        experiment_uuid = None
        if response['status'] == "OK":
            experiment_uuid = response['content']['experiment_uuid']
            self.experiment_uuid = experiment_uuid
            
            self.rpc.data.update({'experiment_uuid': experiment_uuid})
            logging.info('Success to create %s/%s'%(self.project, self.experiment_name))
        else:
            self.experiment_uuid = None
            logging.error('Fail to register experiment in dashboard')


        return experiment_uuid

    def activate(self, project, experiment):
        if project is None or experiment is None:
            logging.error('Must set project and experiment')
            return None

        # 1.step 获得experiment_uuid
        response = self.rpc.experiment.activate.get(experiment_name=experiment, project_name=project)
        if response['status'] == 'OK':
            self.experiment_uuid = response['content']['experiment_uuid']
            self.experiment_name = experiment
            self.project = project
            logging.info('Activate experiment %s/%s'%(response['content']['project_name'], response['content']['experiment_name']))
            # 更新rpc的实验标识字段
            self.rpc.data.update({'experiment_uuid': self.experiment_uuid})
            return self.experiment_uuid
        else:
            logging.info('Couldnt activate %s/%s'%(project, experiment))
            return None

    def list(self, project=None):
        response = self.rpc.experiment.list.get(project_name=project)
        if response['status'] == 'OK':
            return response['content']['experiment_list']

        return []

    def configure(self, *args, **kwargs):
        self.dashboard_ip = kwargs.get('dashboard_ip', self.dashboard_ip)
        self.dashboard_port = kwargs.get('dashboard_port', self.dashboard_port)
        
        token = kwargs.get('token', self.token)
        if token is not None:
            self.token = token

        experiment_uuid = kwargs.get('experiment_uuid', self.experiment_uuid)
        if experiment_uuid is not None:
            self.experiment_uuid = experiment_uuid

        server = kwargs.get('server', self.server)
        if server is not None:
            self.server = server
          
        self.project = kwargs.get('project', self.project)
        self.experiment_name = kwargs.get('experiment', self.experiment_name)
        
    def update(self):
        # update dashboard
        self.job.update()
    
    def exit(self):
        # exit dashboard
        # exit resource
        self.job.exit()

        # stop flag
        self.rpc.experiment.stop.post()

    def error(self):
        # exit resource
        self.job.exit()

        # error flag
        self.rpc.experiment.error.post()

    def __getattr__(self, key):
        if key in ['experiment', 'task', 'apply', 'dataset', 'challenge', 'benchmark']:
            return getattr(self.rpc, key)
        
        return self.__dict__.get(key)


# -*- coding: UTF-8 -*-
# @Time    : 2020-05-29 17:41
# @File    : utils.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from datetime import datetime
from binascii import b2a_hex
import errno
import hashlib
from hmac import compare_digest
import os
import socket
from threading import Thread
import uuid
import warnings
import re
import sys
if sys.version > '3':
    PY3 = True
else:
    PY3 = False


def new_token(*args, **kwargs):
    """generator for new random tokens

    For now, just UUIDs.
    """
    return uuid.uuid4().hex


def hash_token(token, salt=8, rounds=16384, algorithm='sha512'):
    """hash a token, and return it as `algorithm:salt:hash`

    If `salt` is an integer, a random salt of that many bytes will be used.
    """
    h = hashlib.new(algorithm)
    if isinstance(salt, int):
        salt = b2a_hex(os.urandom(salt))
    if isinstance(salt, bytes):
        bsalt = salt
        salt = salt.decode('utf8')
    else:
        bsalt = salt.encode('utf8')
    btoken = token.encode('utf8', 'replace')
    h.update(bsalt)
    for i in range(rounds):
        h.update(btoken)
    digest = h.hexdigest()
    
    return "{algorithm}:{rounds}:{salt}:{digest}".format(**locals())


def compare_token(compare, token):
    """compare a token with a hashed token

    uses the same algorithm and salt of the hashed token for comparison
    """
    algorithm, srounds, salt, _ = compare.split(':')
    hashed = hash_token(token, salt=salt, rounds=int(srounds), algorithm=algorithm).encode('utf8')
    compare = compare.encode('utf8')
    if compare_digest(compare, hashed):
        return True
    return False


def url_path_join(*pieces):
    """Join components of url into a relative url

    Use to prevent double slash when joining subpath. This will leave the
    initial and final / in place

    Copied from notebook.utils.url_path_join
    """
    initial = pieces[0].startswith('/')
    final = pieces[-1].endswith('/')
    stripped = [s.strip('/') for s in pieces]
    result = '/'.join(s for s in stripped if s)
    
    if initial:
        result = '/' + result
    if final:
        result = result + '/'
    if result == '//':
        result = '/'
    
    return result


def timestamp():
  now_time = datetime.now()
  if PY3:
    return now_time.timestamp()
  else:
    epoch = datetime.utcfromtimestamp(0)
    total_seconds = (now_time - epoch).total_seconds()
    # total_seconds will be in decimals (millisecond precision)
    return total_seconds


# -*- coding: UTF-8 -*-
# @Time    : 2020-06-23 12:58
# @File    : __init__.py.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from antvis.client.dashboard import *
from antvis.client.mlogger.container import *
from antvis.client.mlogger.metric import *
from antvis.client.mlogger.complex import *
from antvis.client.mlogger.monitor import *
from antvis.client.mlogger.file import *
from contextlib import contextmanager
import git
import logging


class __Env(object):
    def __init__(self, project=None, experiment=None, token=None, **kwargs):
        self.dashboard = Dashboard(token=token, **kwargs)
        if project is not None and experiment is not None:
            self.dashboard.create_project(project=project, experiment=experiment)
        
        self._webdav_host = None
        self._webdav_login = None
        self._webdav_password = None
        self._webdav_root = ''
        self.use_webdav = False
        
    def create_chart(self, *args, **kwargs):
        return self.dashboard.create_chart(*args, **kwargs)
        
    def create_channel(self, *args, **kwargs):
        return self.dashboard.create_channel(*args, **kwargs)
    
    @property
    def webdav_host(self):
        return self._webdav_host
    
    @property
    def webdav_login(self):
        return self._webdav_login
    
    @property
    def webdav_password(self):
        return self._webdav_password
    
    @property
    def webdav_root(self):
        return self._webdav_root
    
    def webdav(self, host, login, password, root=''):
        self._webdav_host = host
        self._webdav_login = login
        self._webdav_password = password
        self._webdav_root = root
        self.use_webdav = True
    
        
__env = None


def getEnv():
    global __env
    return __env


def config(project=None, experiment=None, token=None, **kwargs):
    global __env
    __env = __Env(project, experiment, token, **kwargs)
    
    
def update():
    global __env
    if __env.dashboard is not None:
        __env.dashboard.update()
    
    
def exit():
    global __env
    if __env.dashboard is not None:
        __env.dashboard.exit()


def error():
    global __env
    if __env.dashboard is not None:
        __env.dashboard.error()


def create_project(project, experiment):
    # 创建实验
    global __env
    experiment_uuid = __env.dashboard.create_project(project=project, experiment=experiment)
    return experiment_uuid


def activate(project, experiment):
    # 激活指定实验
    global __env
    experiment_uuid = __env.dashboard.activate(project=project, experiment=experiment)
    return experiment_uuid


def list(project=None):
    # 获得实验列表
    global __env
    experiment_list = __env.dashboard.list(project=project)
    return experiment_list


@contextmanager
def Context(ip, port, project, experiment, token=None, path='.'):
    # 创建实验基本信息
    global __env
    global tag
    # 配置平台
    assert(project is not None and experiment is not None)
    config(ip, port, project, experiment, token)

    # 获取git信息
    git_commit = ''
    git_branch = ''
    try:
        repo = git.Repo(path)
        git_branch = repo.active_branch.name
        commits = list(repo.iter_commits(git_branch, max_count=1))
        if len(commits) > 0:
            git_commit = commits[0]
    except:
        logging.error('Project %s experiment %s no git info'%(project, experiment))

    # 配置实验信息
    tag.source.name = ''                # source identify (git URL, source file name, notebook name)
    tag.source.type = 'UNKOWN'          # source type. NOTEBOOK,PROJECT,UNKOWN
    tag.source.git.commit = git_commit  # commit hash of excuted code
    tag.source.git.branch = git_branch  # name of branch of excuted code
    tag.source.git.repourl = ''         # URL that executed code was cloned from
    tag.project.env = ''                # runtime context docker / conda
    tag.project.entrypoint = ''         # name of project entry point associated with experiment run
    tag.docker.image.name = ''          # name of docker image used to execute experiment run
    tag.docker.image.id = ''            # id of docker image used to execute experiment run
    tag.user = ''                       # identify of user who created experiment run
    tag.note.content = ''               # a descriptive note about experiment run
    tag.runname = ''                    # human readable name that identify experiment run

    yield

    # 更新&退出
    update()


class Tag(object):
    def __init__(self, parent='', parent_tag=None):
        self._tags = {}
        self._parent = parent
        self._parent_tag = parent_tag
        self._value = None

    def __setattr__(self, key, value):
        if key.startswith('_'):
            return super(Tag, self).__setattr__(key, value)

        self.__dict__['_tags'][key] = Tag('%s/%s'%(self._parent, key), self)
        value_str = str(value)
        self._value = value_str
        self.__update('%s/%s'%(self._parent, key), value_str)

    def __getattr__(self, item):
        if item == 'get':
            def _func():
                if self._parent_tag.__dict__['_value'] is not None:
                    return self._parent_tag.__dict__['_value']

                data = mlogger.getEnv().dashboard.rpc.experiment.tag.get(tag=self._parent)
                if data['status'] == 'OK':
                    return data['content']['value']

                return ''
            return _func

        if item not in self._tags:
            self.__dict__['_tags'][item] = Tag('%s/%s'%(self._parent, item), self)

        return self._tags[item]

    def __update(self, key, value):
        getEnv().dashboard.job.cache({'TAG': {key: value}})


tag = Tag()


# encoding=utf-8
# Time: 8/28/17
# File: job.py
# Author: jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function

import uuid
import threading
import numpy as np
import re
import copy
import sys
import scipy.misc
import base64
import os
import logging
import requests
from antvis.utils.utils import *
from antvis.utils.encode import *
import json
from PIL import Image
import zlib

PYTHON_VERSION = sys.version_info[0]
if PYTHON_VERSION == 2:
    import Queue as queue
elif PYTHON_VERSION == 3:
    import queue as queue


class Chart(object):
  def __init__(self, title="chart", x_axis="x", y_axis="y", dashboard=None):
    self.chart_title = title
    self.chart_x_axis = x_axis
    self.chart_y_axis = y_axis
    self.chart_channels = []
    self.chart_map = {}
    self.dashboard = dashboard

  @property
  def uuid(self):
    if self.dashboard.experiment_stage not in self.chart_map:
      self.chart_map[self.dashboard.experiment_stage] = unicode(uuid.uuid1()) if PYTHON_VERSION == 2 else str(uuid.uuid1())
    return self.chart_map[self.dashboard.experiment_stage]

  @property
  def title(self):
    return self.chart_title

  @property
  def x_axis(self):
    return self.chart_x_axis

  @x_axis.setter
  def x_axis(self, val):
    self.chart_x_axis = val

  @property
  def y_axis(self):
    return self.chart_y_axis

  @y_axis.setter
  def y_axis(self, val):
    self.chart_y_axis = val

  @property
  def channels_num(self):
    return len(self.chart_channels)

  def bind_channel(self, channel):
    if len(self.chart_channels) != 0:
      assert(channel.channel_type == self.chart_channels[0].channel_type)

    # 检查是否支持multi-channel
    if len(self.chart_channels) > 0 and channel.channel_type not in ['SCATTER', 'LINE', 'HISTOGRAM', 'BAR', 'TEXT', 'IMAGE']:
      logging.error('channel type %s not support multi-channel in one chart'%channel.channel_type)
      raise NotImplementedError
      
    channel.id = len(self.chart_channels)
    channel.chart = self
    self.chart_channels.append(channel)
    
  
class Channel(object):
  def __init__(self, channel_name=None, channel_type=None, channel_job=None, time_series=True, **channel_params):
    self.channel_id = -1
    self.channel_name = channel_name
    self.channel_type = channel_type
    self.channel_chart = None
    self.channel_job = channel_job
    self.channel_params = channel_params
    self.time_series = time_series
    assert(self.channel_type in ["IMAGE", "SVG", "SCATTER", "HISTOGRAM", "BAR", "LINE", "TEXT", "TABLE", "HEATMAP"])

  @property
  def params(self):
    return self.channel_params

  @property
  def chart(self):
    return self.channel_chart
  
  @chart.setter
  def chart(self, val):
    self.channel_chart = val

  @property
  def id(self):
    return self.channel_id
  
  @id.setter
  def id(self, val):
    self.channel_id = val

  @property
  def name(self):
    return self.channel_name
  
  @name.setter
  def name(self, val):
    self.channel_name = val

  def transform_to_svg(self, data):
    x, y = data
    return x, y
  
  def transform_to_image(self, data):
    data_x, data_y = data
    try:
      data_x = float(data_x)
    except:
      logging.error("Channel X Must be Scalar Data")
      return None

    try:
      if len(data_y.shape) != 2 and len(data_y.shape) != 3:
        logging.error("Channel Y Must be 2 or 3 Dimension")
        return None
      if len(data_y.shape) == 3:
        if data_y.shape[2] != 3:
          logging.error("Channel Y Must Possess 3 or 1 Channels")
          return None
      if data_y.dtype != np.uint8:
        logging.error('Image type must be np.uint8')
        return None

      return (data_x, base64.b64encode(png_encode(data_y)).decode('utf-8'))
    except:
      logging.error("Channel Y Must be Numpy Array")

  def transform_to_scatter(self, data):
    data_x, data_y = data
    try:
      data_x = float(data_x)
    except:
      logging.error("X Must be Scalar Data")
      return None

    if type(data_y) == dict:
      try:
        data_y['y'] = float(data_y['y'])
        data_y['size'] = float(data_y['size'])
        return (data_x, data_y)
      except:
        logging.error("Y mean and var must be Scalar Data")

    try:
      data_y = float(data_y)
    except:
      logging.error("Y Must be Scalar Data")
    return (data_x, {'y': data_y, 'size': 10})
  
  def transform_to_line(self, data):
    data_x, data_y = data
    try:
      data_x = float(data_x)
    except:
      logging.error("X Must be Scalar Data")
      return None
    
    if type(data_y) == dict:
      try:
        data_y['mean'] = float(data_y['mean'])
        data_y['var'] = float(data_y['var'])
        return (data_x, data_y)
      except:
        logging.error("Y mean and var must be Scalar Data")
    
    try:
      data_y = float(data_y)
    except:
      logging.error("Y Must be Scalar Data")
    return (data_x, {'mean': data_y, 'var': -1.0})

  def transform_to_bar(self, data):
    data_x, data_y = data
    if type(data_x) != list or type(data_y) != list:
      return None

    if len(data_x) != len(data_y):
      return None

    return data_x, data_y

  def transform_to_histogram(self, data):
    data_x, data_y = data
    try:
      data_x = float(data_x)
    except:
      logging.error("Channel X Must be Scalar Data")
      return None
    
    try:
      data_y = np.array(data_y)
      data_y = data_y.flatten()
      bins = 10 # default bins
      if "BINS" in self.params:
        bins = self.params['BINS']
      
      data_y = np.histogram(data_y, bins)
    except:
      logging.error("Channel Y Must be Numpy Array")
    return (data_y[1].tolist(),data_y[0].tolist())

  def update(self, x=0, y=0):
    # {"CHART", (chart_id, chart_title,...)}
    x = copy.deepcopy(x)
    y = copy.deepcopy(y)

    data_vis_type = self.channel_type
    if self.channel_type == 'HISTOGRAM':
      data_vis_type = 'BAR'

    data = {"CHART": {"chart_uuid": self.chart.uuid,
                               "chart_title": self.chart.title,
                               "chart_x_axis": self.chart.x_axis,
                               "chart_y_axis": self.chart.y_axis,
                               "chart_type": data_vis_type,
                               "chart_channels": self.chart.channels_num,
                               "channel_id": self.id,
                               "channel_type": data_vis_type,
                               "channel_name": self.channel_name,
                               "channel_time_series": self.time_series,
                               "channel_data": []}}
    
    if self.channel_type == 'IMAGE':
      xxyy = self.transform_to_image((x,y))
      if xxyy is None:
        return
      
      x,y = xxyy
      data['CHART']['channel_data'].append({'x': x, 'y': y})
    elif self.channel_type == 'SVG':
      xxyy = self.transform_to_svg((x,y))
      if xxyy is None:
        return
      
      x,y = xxyy
      data['CHART']['channel_data'].append({'x': x, 'y': y})
    elif self.channel_type == 'SCATTER':
      if not self.time_series:
        data['CHART']['channel_data'] = []
        for x, y in zip(x, y):
          xxyy = self.transform_to_scatter((x,y))
          if xxyy is None:
            continue

          data['CHART']['channel_data'].append({'x': xxyy[0], 'y': xxyy[1]})

        self.channel_job.cache(data)
        return

      xxyy = self.transform_to_scatter((x,y))
      if xxyy is None:
        return
    
      x,y = xxyy
      data['CHART']['channel_data'].append({'x': x, 'y': y})
    elif self.channel_type == 'LINE':
      if not self.time_series:
        data['CHART']['channel_data'] = []
        for x, y in zip(x, y):
          xxyy = self.transform_to_scatter((x,y))
          if xxyy is None:
            continue

          data['CHART']['channel_data'].append({'x': xxyy[0], 'y': xxyy[1]})

        self.channel_job.cache(data)
        return

      xxyy = self.transform_to_line((x, y))
      if xxyy is None:
        return
  
      x, y = xxyy
      data['CHART']['channel_data'].append({'x': x, 'y': y})
    elif self.channel_type == 'HISTOGRAM':
      xxyy = self.transform_to_histogram((x, y))
      if xxyy is None:
        return
      
      x,y = xxyy
      data['CHART']['channel_data'].append({'x': x, 'y': y})
    elif self.channel_type == 'BAR':
      xxyy = self.transform_to_bar((x, y))
      if xxyy is None:
        return

      x, y = xxyy
      data['CHART']['channel_data'].append({'x': x, 'y': y})
    elif self.channel_type == 'HEATMAP':
      if type(y) == np.ndarray:
        y = y.tolist()
      
      if type(y) != list:
        return

      data['CHART']['channel_data'].append({'x': x, 'y': y})
    else:
      data['CHART']['channel_data'].append({'x': x, 'y': y})
    
    # push to cache
    self.channel_job.cache(data)


class ExitSig(object):
  pass


class Job(threading.Thread):
  def __init__(self, dashboard=None, cache_max_size=100):
    super(Job, self).__init__()
    self.data_queue = queue.Queue()
    self.dashboard = dashboard
    self.pid = str(os.getpid())
    self.charts = []
    self.cache_data = []
    self.cache_max_size = cache_max_size
    self.dashboard_ip = self.dashboard.dashboard_ip
    self.dashboard_port = self.dashboard.dashboard_port
    self.dashboard_prefix = 'http'
    self.daemon = True
  
  def create_channel(self, channel_name, channel_type, **kwargs):
    return Channel(channel_name, channel_type, self, **kwargs)

  def create_chart(self, chart_channels, chart_title, chart_x_axis="x", chart_y_axis="y", **kwargs):
    chart = Chart(chart_title, chart_x_axis, chart_y_axis, dashboard=self.dashboard)
    self.charts.append(chart)
    channel_type = None
    for cc in chart_channels:
      if channel_type is None:
        channel_type = cc.channel_type
      else:
        # assert channel has the same type
        assert(channel_type == cc.channel_type)

      chart.bind_channel(cc)
    return chart
  
  def cache(self, data):
    if self.dashboard.experiment_name is not None:
      # fill context data
      context_data = {'APP_TIME': self.dashboard.launch_time,
                      'APP_NOW_TIME': timestamp(),
                      'APP_SERVER': self.dashboard.server,
                      'APP_EXPERIMENT_NAME': self.dashboard.experiment_name,
                      'APP_EXPERIMENT_UUID': self.dashboard.experiment_uuid,
                      'APP_HYPER_PARAMETER': self.dashboard.experiment_hyper_parameter,
                      'APP_STAGE': self.dashboard.experiment_stage}
  
      data.update(context_data)
      
      # cache data
      self.cache_data.append(data)
      
      # auto triger update data
      if len(self.cache_data) > self.cache_max_size:
        self.update()
  
  def update(self):
    if len(self.cache_data) > 0:
      self.data_queue.put(self.cache_data)
      self.cache_data = []

  def exit(self):
    self.data_queue.put(ExitSig())
    self.join()
  
  def run(self):
    while True:
      # 0.step get data
      data = self.data_queue.get()

      # 1.step is exit signal
      if type(data) == ExitSig:
        break

      # 2.step data is none or empty
      if data is None or len(data) == 0:
          continue
      
      # 3.step merge data by chart uuid
      data_package = {'CHART': {}, 'APP_STAGE': data[0]['APP_STAGE']}
      for item in data:
        if 'TAG' in item:
          if 'TAG' not in data_package:
            data_package["TAG"] = {}
          data_package['TAG'].update(item['TAG'])
        elif 'CHART' in item:
          chart_uuid = item['CHART']['chart_uuid']
          channel_id = item['CHART']['channel_id']

          if chart_uuid not in data_package['CHART']:
            data_package['CHART'][chart_uuid] = {}
            data_package['CHART'][chart_uuid].update(item['CHART'])
            data_package['CHART'][chart_uuid].pop('channel_id')
            data_package['CHART'][chart_uuid].pop('channel_type')
            data_package['CHART'][chart_uuid].pop('channel_name')
            data_package['CHART'][chart_uuid].pop('channel_data')
            data_package['CHART'][chart_uuid]['channels'] = {}

          if channel_id not in data_package['CHART'][chart_uuid]['channels']:
            data_package['CHART'][chart_uuid]['channels'][channel_id] = None

          if data_package['CHART'][chart_uuid]['channels'][channel_id] is None:
            data_package['CHART'][chart_uuid]['channels'][channel_id] = {}
            data_package['CHART'][chart_uuid]['channels'][channel_id]['channel_id'] = item['CHART']['channel_id']
            data_package['CHART'][chart_uuid]['channels'][channel_id]['channel_type'] = item['CHART']['channel_type']
            data_package['CHART'][chart_uuid]['channels'][channel_id]['channel_name'] = item['CHART']['channel_name']
            data_package['CHART'][chart_uuid]['channels'][channel_id]['channel_data'] = item['CHART']['channel_data']
            data_package['CHART'][chart_uuid]['channels'][channel_id]['channel_time_series'] = item['CHART']['channel_time_series']
          else:
            if item['CHART']['channel_time_series']:
              data_package['CHART'][chart_uuid]['channels'][channel_id]['channel_data'].extend(item['CHART']['channel_data'])
            else:
              data_package['CHART'][chart_uuid]['channels'][channel_id]['channel_data'] = item['CHART']['channel_data']

          if data_package['CHART'][chart_uuid]['chart_channels'] < len(data_package['CHART'][chart_uuid]['channels']):
            data_package['CHART'][chart_uuid]['chart_channels'] = len(data_package['CHART'][chart_uuid]['channels'])

      # 4.step remote call
      experiment_data = \
        {'experiment_name': self.dashboard.experiment_name,
         'experiment_uuid': self.dashboard.experiment_uuid,
         'experiment_data': zlib.compress(json.dumps(data_package).encode()),
         'experiment_stage': data[0]['APP_STAGE'],
         'experiment_hyper_parameter': data[0]['APP_HYPER_PARAMETER']}

      if self.dashboard != None:
        self.dashboard.rpc.experiment.patch(**experiment_data)



from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function

import antvis.client.mlogger as mlogger
import numpy as np
import time


if __name__ == "__main__":
  # 1.step 配置日志环境
  mlogger.config('127.0.0.1',
                 8999,
                 'det',
                 'b',
                 token='3bdbf5aa1c274b05971bdf2b58508816')

  # 2.step 设置实验标签
  mlogger.tag.learning_rate = '0.01'
  mlogger.tag.batch_size = '126'
  mlogger.tag.optimizer = 'SGD'
  mlogger.tag.weight_decay = '0.0001'

  # 2.step 配置容器
  container_1 = mlogger.Container()
  container_2 = mlogger.Container()

  # 2.1.step 配置绘制图表——Simple
  container_1.growth_curve = mlogger.metric.Simple('growth')

  for _ in range(50):
    container_1.growth_curve.update(_)
  mlogger.update()

  # 2.2.step 配置绘制图表——Image
  container_1.image_show_1 = mlogger.complex.Image('image-1')
  container_1.image_show_2 = mlogger.complex.Image('image-2')

  # 2.3.step 配置绘制图表——Histogram
  container_1.check_his = mlogger.complex.Histogram('check-his', BINS=200)

  for _ in range(5):
    container_1.image_show_1.update(np.random.randint(0, 255, (255, 255, 3)))
    container_1.image_show_2.update(np.random.random((512, 256, 3)))
    container_1.check_his.update(np.random.random((50, 50)))
  mlogger.update()

  # 2.4.step 配置绘制图表——Bar
  container_1.check_bar_1 = mlogger.complex.Bar('check-bar-1')
  container_1.check_bar_1.update([5,6,7,8], [10,40,100,1])

  container_1.check_bar_2 = mlogger.complex.Bar('check-bar-2')
  container_1.check_bar_2.update(['a','b','c','d'], [1,2,3,4])
  mlogger.update()

  # 2.5.step 配置绘制图表——Heatmap
  container_2.check_heatmap = mlogger.complex.Heatmap('check-heatmap')
  container_2.check_heatmap.update(np.random.randint(0, 255, (500, 500)))
  mlogger.update()
  print('sdf')

  # 2.6.step 配置绘制图表——Text
  container_2.check_text = mlogger.complex.Text('check-text')

  # 2.7.step 配置绘制图表——Table
  container_2.check_table = mlogger.complex.Table('check-table')

  # 2.8.step 配置绘制图表——Scatter
  container_2.check_scatter = mlogger.complex.Scatter('check-scatter')

  # 2.9.step 监控
  container_2.cpu = mlogger.CPU('train process')
  container_2.mem = mlogger.Memory('train process-mem')

  for _ in range(10):
    container_2.check_heatmap.update(np.random.randint(0, 255, (500, 500)))
    container_2.check_text.update('hello %d'%_)
    container_2.check_table.r0c0 = 'A'
    container_2.check_table.r2c2 = 'C'
    container_2.check_table.update()

    container_2.check_scatter.update([np.random.randint(0, 10, [10]).tolist(),
                            np.random.randint(0, 10, [10]).tolist()])

    container_2.cpu.update()
    container_2.mem.update()
    time.sleep(1)
    mlogger.update()



# -*- coding: UTF-8 -*-
# @Time    : 2020/7/23 7:33 下午
# @File    : __init__.py.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from antvis.client import mlogger
import os
import tarfile
import json
import uuid
from antvis.client.mlogger.metric.base import *
import sys
import logging


class FileLogger(object):
    def __init__(self, title):
        self.group_name = ''
        self.tag = ''
        self.title = title
        assert title != ''

        # experiment config
        self.project = mlogger.getEnv().dashboard.project
        self.experiment_name = mlogger.getEnv().dashboard.experiment_name
        self.experiment_uuid = mlogger.getEnv().dashboard.experiment_uuid
        self.folder = ''

        if self.experiment_uuid is None:
            return

    @property
    def name(self):
        return self.title

    @name.setter
    def name(self, val):
        group_name = ''
        self.tag = val
        if '/' in val:
            group_name, self.tag = val.split('/')
        self.group_name = group_name

    def update(self, *args, **kwargs):
        # 支持WEBDAV, MLTALKER
        # antvis/experiment_uuid/group/%s_(latest_%d).xxx
        if len(args) == 0:
            return

        local_file_path = os.path.normpath(args[0])
        if not os.path.exists(local_file_path):
            return
        # 扩展名
        ext_name = local_file_path.split('.')[-1]
        # 文件名
        file_name = local_file_path.split('/')[-1]

        # 单位M
        fsize = os.path.getsize(local_file_path)
        fsize = fsize / float(1024 * 1024)
        fsize = round(fsize, 2)

        # 1.step 获得临时秘钥
        response = \
          mlogger.getEnv().dashboard.rpc.cos.experiment.get(file_size=fsize,
                                                            operator='upload')
        if response['status'] == 'ERROR':
          logging.error('could get cos credential')
          return

        tmpSecretId = response['content']['tmpSecretId']
        tmpSecretKey = response['content']['tmpSecretKey']
        sessionToken = response['content']['sessionToken']
        region = response['content']['region']
        bucket = response['content']['bucket']

        # 2.step 上传文件
        tag_str = self.tag if self.group_name == '' else '%s/%s' % (self.group_name, self.tag)
        try:
          config = CosConfig(Region=region,
                             SecretId=tmpSecretId,
                             SecretKey=tmpSecretKey,
                             Token=sessionToken,
                             Scheme='https')
          client = CosS3Client(config)
          with open(local_file_path, 'rb') as fp:
            client.put_object(
                Bucket=bucket,
                Body=fp,
                Key='{}/{}/{}/{}'.format(self.experiment_uuid, tag_str, self.title, file_name),
                StorageClass='STANDARD',
                EnableMD5=False
              )
        except:
          logging.error('upload {} error'.format(local_file_path))
          return

        # 3.step 更新平台记录
        mlogger.getEnv().dashboard.experiment.patch(**{
            'experiment_name': self.experiment_name,
            'experiment_uuid': self.experiment_uuid,
            'experiment_stage': mlogger.getEnv().dashboard.experiment_stage,
            'experiment_data': json.dumps({'FILE_ABSTRACT': {
                'group': tag_str,
                'title': self.title,
                'backend': 'ANTVIS',
                'path': '{}/{}/{}/{}'.format(self.experiment_uuid, tag_str, self.title, file_name),
                'size': fsize,
            }, 'APP_STAGE': mlogger.getEnv().dashboard.experiment_stage})
        })

    def config(self, **kwargs):
        pass

    def get(self):
        # 文件下载
        # 1.step 获取日志文件
        tag_str = self.tag if self.group_name == '' else '%s/%s' % (self.group_name, self.tag)
        response = \
            mlogger.getEnv().dashboard.\
                rpc.experiment.file.get(experiment_uuid=self.experiment_uuid,
                                        key='{}/{}'.format(tag_str, self.title))
        if response['status'] == 'ERROR':
          logging.error('couldnt get log file path')
          return

        log_file = response['content']['path']
        log_name = log_file.split('/')[-1]
        tmpSecretId = response['content']['tmpSecretId']
        tmpSecretKey = response['content']['tmpSecretKey']
        sessionToken = response['content']['sessionToken']
        region = response['content']['region']
        bucket = response['content']['bucket']

        # 2.step 下载日志文件
        try:
          config = CosConfig(Region=region,
                             SecretId=tmpSecretId,
                             SecretKey=tmpSecretKey,
                             Token=sessionToken,
                             Scheme='https')
          client = CosS3Client(config)
          response = client.get_object(Bucket=bucket, Key=log_file)
          file_content = response['Body'].get_raw_stream()
          with open('./{}'.format(log_name), 'wb') as fp:
            fp.write(file_content.read())
        except:
          logging.error('download {} error'.format(log_file))


class FolderLogger(FileLogger):
    def __init__(self, title):
        super(FolderLogger, self).__init__(title)
        self.group_name = ''
        self.file_title = title
        assert title != ''

    def update(self, *args, **kwargs):
        # antvis/experiment_uuid/group/%s_(latest_%d).xxx
        if len(args) == 0:
            return

        local_folder_path = args[0]
        if not os.path.exists(local_folder_path):
            return

        if not os.path.isdir(local_folder_path):
            return

        # 压缩打包
        tar_file_name = '%s.tar.gz' % (self.file_title)
        tar = tarfile.open('./'+tar_file_name, 'w:gz')
        for root, dirs, files in os.walk(local_folder_path):
            rel_root = os.path.relpath(root, local_folder_path)
            for f in files:
                tar.add(os.path.join(root, f), arcname=os.path.join(rel_root, f))
        tar.close()

        # 上传
        super(FolderLogger, self).update('./'+tar_file_name)

        # 清空临时压缩包
        os.remove('./'+tar_file_name)


# -*- coding: UTF-8 -*-
# @Time    : 2020-05-28 22:37
# @File    : app.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
import time
import numpy as np
import json
import antvis.client.mlogger as mlogger
# import tensorflow as tf
# import antvis.client.mlogger.tensorflow
# import io
# import matplotlib.pyplot as plt
# import os
# import re
# import torch

mlogger.config('127.0.0.1', 8999, token='42fb617800384df983edbd9509c2b2ba')
mlogger.activate('XX', 'exp.3')
xp = mlogger.Container()
xp.ff = mlogger.FileLogger('model file')
xp.ff.get()


# mlogger.config('127.0.0.1', 8999, 'XX', 'exp', token='42fb617800384df983edbd9509c2b2ba')
#
# # 上传1G文件
# xp = mlogger.Container()
# xp.ff = mlogger.FileLogger('model file')
# xp.ff.update('/Users/jian/Downloads/RR/pycharm-community-2020.2.1.dmg')
# mlogger.exit()



# # xp = mlogger.Container()
# # xp.tt = mlogger.tensorflow.Image('TT')
# # a = tf.random.normal((1, 100,100, 3))
# # b = tf.random.normal((1, 100,100, 3))
# # c = a*255 + b*255
# # c = tf.cast(c,tf.uint8)
# # d = xp.tt(c)
# #
# # with tf.Session() as sess:
# #     value = sess.run(d)
# #     print(value)
# #
# #
# # mlogger.update()
# # mlogger.exit()
#
# # # 1. 获得实验项目列表
# # print(mlogger.list())
# #
# # # 2. 激活实验
# # mlogger.activate('you', 'jian10')
# #
# # # 3. 创建实验
# # # mlogger.create_project('you', 'jian10')
# # # mlogger.activate('you', 'DD')
# #
# # 4. 添加可视化数据
# # 4.1 文本数据
# xp = mlogger.Container()
# xp.train = mlogger.Container()
# xp.train.log_1 = mlogger.complex.Text('T2')
# xp.train.log_2 = mlogger.complex.Text('T2')
#
# for _ in range(50):
#     xp.train.log_1.info('hello')
#     xp.train.log_2.warn('world')
# mlogger.update()
#
#
# # print(xp.train.log_1.get())
# # print(xp.train.log_2.get())
#
# xp.ss = mlogger.complex.Image("Save")
# xp.ss.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.ss.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.ss.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.ss.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.ss.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.ss.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.tt = mlogger.complex.Image("Save")
# xp.tt.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.tt.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.qq = mlogger.complex.Image("Save")
# xp.qq.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.qq.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
#
# xp.qq1 = mlogger.complex.Image("Save")
# xp.qq1.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.qq1.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
#
# xp.qq2 = mlogger.complex.Image("Save")
# xp.qq2.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.qq2.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
#
# xp.qq3 = mlogger.complex.Image("Save")
# xp.qq3.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.qq3.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
#
# xp.qq4 = mlogger.complex.Image("Save")
# xp.qq4.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
# xp.qq4.update(np.random.randint(0,255,[100,100,3], dtype=np.uint8))
#
#
# mlogger.update()
# # 4.2 时序数据
# xp.step = mlogger.metric.Simple('global step')
#
# for i in range(10):
#     xp.step.update(i)
# mlogger.update()
#
# # 4.3 散点图
# xp.pareto_front = mlogger.complex.Scatter('pareto front')
# xp.pareto_front.update([np.random.randint(0,10,[10]).tolist(),
#                         np.random.randint(0,10,[10]).tolist()])
#
# # 4.4 直方图
# xp.mm = mlogger.complex.Histogram('EE')
# xp.mm.update(np.random.random((10, 30)))
# mlogger.update()
# print('his')
# # print(xp.mm.get())
#
# # 4.5 柱状图
# xp.dis = mlogger.complex.Bar('DD')
# xp.dis.update(['x','y','z'],[10,100,9])
# mlogger.update()
#
# # 4.6 热力图
# xp.heat = mlogger.complex.Heatmap('H')
# xp.heat.update(np.random.randint(0,255,(500,500)))
# mlogger.update()
#
# # 4.7 表格数据
# xp.cc = mlogger.complex.Table("SO")
# xp.cc.r0c0 = 'A'
# xp.cc.r0c1 = 'B'
# xp.cc.r1c0 = 'c'
# xp.cc.r1c1 = 'D'
# xp.cc.update()
# mlogger.update()
# # print(xp.cc.get())
#
# # 4.8 标签
# mlogger.tag.learning_rate = '0.01'
# mlogger.tag.batch_size = '126'
# mlogger.tag.optimizer = 'SGD'
# mlogger.tag.weight_decay = '0.0001'
# mlogger.update()
# # mlogger.tag.cc.get()
#
# # # 4.9 文件
# # xp.ff = mlogger.FileLogger('model file')
# # xp.ff.update('/Users/jian/Downloads/TT/index.md')
# #
# # # 4.10 文件夹
# # xp.mm = mlogger.FolderLogger('fo')
# # xp.mm.update('/Users/jian/Downloads/TT/')
#
# # 4.11 环境监控
# xp.cpu = mlogger.CPU('train process')
# for _ in range(10):
#     xp.cpu.update()
# mlogger.update()
# # print(xp.cpu.get())
#
# xp.mem = mlogger.Memory('train process-mem')
# for _ in range(10):
#     xp.mem.update()
# mlogger.update()
#
# # 4.12 git环境监控
# mlogger.exit()
# #
# # # # 获得记录数据
# # # print(xp.train.log_1.get())
# # # print(xp.train.log_2.get())
# #
# # # 4.2 时序数据
# # xp.step = mlogger.metric.Simple('global step')
# #
# # for i in range(10):
# #     xp.step.update(i)
# # mlogger.update()
# # # print('xp step')
# # # print(xp.step.get())
# #
# # # 4.3 散点图
# # xp.pareto_front = mlogger.complex.Scatter('pareto front')
# # xp.pareto_front.update([np.random.randint(0,10,[10]).tolist(),
# #                         np.random.randint(0,10,[10]).tolist()])
# # mlogger.update()
# # # print('xp pareto front')
# # # print(xp.pareto_front.get())
# #
# # # 4.4 直方图
# # xp.mm = mlogger.complex.Histogram('EE')
# # xp.mm.update(np.random.random((10, 30)))
# # mlogger.update()
# # # print('histgram')
# # # print(xp.mm.get())
# #
# # # 4.5 柱状图
# # xp.dis = mlogger.complex.Bar('DD')
# # xp.dis.update(['x','y','z'],[10,100,9])
# # mlogger.update()
# # # print('bar')
# # # print(xp.dis.get())
# #
# # # 4.6 热力图
# # xp.heat = mlogger.complex.Heatmap('H')
# # xp.heat.update(np.random.randint(0,255,(500,500)))
# # mlogger.update()
# # # print('heat')
# # # print(xp.heat.get())
# #
# # # 4.7 表格数据
# # xp.cc = mlogger.complex.Table("SO")
# # xp.cc.r0c0 = 'A'
# # xp.cc.r0c1 = 'B'
# # xp.cc.r1c0 = 'c'
# # xp.cc.r1c1 = 'D'
# # xp.cc.update()
# # mlogger.update()
# # # print(xp.cc.get())
# # # 5 文件日志
# # xp.hh = mlogger.FileLogger('xxx')
# # xp.hh.update('/Users/zhangjian52/Downloads/train - 2020-08-12T151311.250.log')
# #
# #
# # # 6 标签
# # # mlogger.tag.A = 'A'
# # mlogger.tag.B.C.D = 'bcn'
# # mlogger.update()
# # print(mlogger.tag.B.C.D.get())



# -*- coding: UTF-8 -*-
# @Time    : 2020/7/23 6:10 下午
# @File    : __init__.py.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from antvis.client.mlogger.metric.base import *
import psutil
import uuid
import re
import os


class CPU(Base):
    def __init__(self, plot_title, **kwargs):
        plot_title = 'monitor/%s'%plot_title
        super(CPU, self).__init__(plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = 'percent'

        self.channel =\
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.channel_min = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.channel_max =\
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)

        self.reset()

    @property
    def name(self):
        return self.channel.channel_name

    @name.setter
    def name(self, val):
        self.channel.channel_name = '%s/avg' % val
        self.channel_min.channel_name = '%s/min' % val
        self.channel_max.channel_name = '%s/max' % val

        group_name = ''
        if '/' in val:
            group_name, _ = val.split('/')

        if self.chart is None:
            group_plot_title = self.plot_title
            if group_name != '':
                group_plot_title = '{}/{}'.format(group_name, self.plot_title)

            if group_plot_title not in Base.charts:
                Base.charts[group_plot_title] = mlogger.getEnv().create_chart([], group_plot_title)
            self.chart = Base.charts[group_plot_title]
            self.chart.chart_x_axis = self.chart_x_axis
            self.chart.chart_y_axis = self.chart_y_axis

        self.chart.bind_channel(self.channel)
        self.chart.bind_channel(self.channel_min)
        self.chart.bind_channel(self.channel_max)

    @property
    def value(self):
        return self._avg

    def reset(self):
        self._avg = None
        self._max_val = -np.inf
        self._min_val = np.inf
        self._total_weight = 0
        return self

    def _update(self, *args, **kwargs):
        if self._avg is None:
            self._avg = 0.0

        # 获取cpu占用率
        val = psutil.cpu_percent(0)

        # 更新
        r = self._total_weight / (1.0 + self._total_weight)
        self._avg = r * self._avg + (1 - r) * val
        self._total_weight += 1.0

        if val > self._max_val:
            self._max_val = val
        if val < self._min_val:
            self._min_val = val

        self.channel.update(self.time, self._avg)
        self.channel_min.update(self.time, self._min_val)
        self.channel_max.update(self.time, self._max_val)


class Memory(Base):
    def __init__(self, plot_title, **kwargs):
        plot_title = 'monitor/%s' % plot_title
        super(Memory, self).__init__(plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = 'percent'

        self.channel = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.channel_min = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.channel_max = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)

        self.reset()

    @property
    def name(self):
        return self.channel.channel_name

    @name.setter
    def name(self, val):
        self.channel.channel_name = '%s/avg' % val
        self.channel_min.channel_name = '%s/min' % val
        self.channel_max.channel_name = '%s/max' % val

        group_name = ''
        if '/' in val:
            group_name, _ = val.split('/')

        if self.chart is None:
            group_plot_title = self.plot_title
            if group_name != '':
                group_plot_title = '{}/{}'.format(group_name, self.plot_title)

            if group_plot_title not in Base.charts:
                Base.charts[group_plot_title] = mlogger.getEnv().create_chart([], group_plot_title)
            self.chart = Base.charts[group_plot_title]
            self.chart.chart_x_axis = self.chart_x_axis
            self.chart.chart_y_axis = self.chart_y_axis

        self.chart.bind_channel(self.channel)
        self.chart.bind_channel(self.channel_min)
        self.chart.bind_channel(self.channel_max)

    @property
    def value(self):
        return self._avg

    def reset(self):
        self._avg = None
        self._max_val = -np.inf
        self._min_val = np.inf
        self._total_weight = 0
        return self

    def _update(self, *args, **kwargs):
        if self._avg is None:
            self._avg = 0.0

        # 获取内存占用率
        val = psutil.virtual_memory().percent

        # 更新
        r = self._total_weight / (1.0 + self._total_weight)
        self._avg = r * self._avg + (1 - r) * val
        self._total_weight += 1.0

        if val > self._max_val:
            self._max_val = val
        if val < self._min_val:
            self._min_val = val

        self.channel.update(self.time, self._avg)
        self.channel_min.update(self.time, self._min_val)
        self.channel_max.update(self.time, self._max_val)


class GPU(Base):
    def __init__(self, plot_title, **kwargs):
        plot_title = 'monitor/%s' % plot_title
        super(GPU, self).__init__(plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = 'percent'

        self.channel = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.channel_min = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.channel_max =\
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)

        self.reset()

    @property
    def name(self):
        return self.channel.channel_name

    @name.setter
    def name(self, val):
        self.channel.channel_name = '%s/avg' % val
        self.channel_min.channel_name = '%s/min' % val
        self.channel_max.channel_name = '%s/max' % val

        group_name = ''
        if '/' in val:
            group_name, _ = val.split('/')

        if self.chart is None:
            group_plot_title = self.plot_title
            if group_name != '':
                group_plot_title = '{}/{}'.format(group_name, self.plot_title)

            if group_plot_title not in Base.charts:
                Base.charts[group_plot_title] = mlogger.getEnv().create_chart([], group_plot_title)
            self.chart = Base.charts[group_plot_title]
            self.chart.chart_x_axis = self.chart_x_axis
            self.chart.chart_y_axis = self.chart_y_axis

        self.chart.bind_channel(self.channel)
        self.chart.bind_channel(self.channel_min)
        self.chart.bind_channel(self.channel_max)

    @property
    def value(self):
        return self._avg

    def reset(self):
        self._avg = None
        self._max_val = -np.inf
        self._min_val = np.inf
        self._total_weight = 0
        return self

    def get_gpu_percent(self):
        try:
            p = os.popen('nvidia-smi')
            content = p.read()
            p.close()
            if content == '':
                return 0.0

            gpu_percent_list = re.findall(r"[0-9]*%", content)
            vv = []
            for g_v in gpu_percent_list:
                vv.append(float(g_v[:-1]))

            value = np.mean(np.array(vv))
            value = float(value)
            return value
        except:
            return 0.0

    def _update(self, *args, **kwargs):
        if self._avg is None:
            self._avg = 0.0

        # 获取内存占用率
        val = self.get_gpu_percent()

        # 更新
        r = self._total_weight / (1.0 + self._total_weight)
        self._avg = r * self._avg + (1 - r) * val
        self._total_weight += 1.0

        if val > self._max_val:
            self._max_val = val
        if val < self._min_val:
            self._min_val = val

        self.channel.update(self.time, self._avg)
        self.channel_min.update(self.time, self._min_val)
        self.channel_max.update(self.time, self._max_val)


class GPUMemory(Base):
    def __init__(self, plot_title, **kwargs):
        plot_title = 'monitor/%s' % plot_title
        super(GPUMemory, self).__init__(plot_title, **kwargs)
        self.chart_x_axis = 'time'
        self.chart_y_axis = 'MB'

        self.channel = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.channel_min = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)
        self.channel_max = \
            mlogger.getEnv().create_channel(str(uuid.uuid4()),
                                            channel_type='LINE',
                                            **self.channel_config)

        self.reset()

    @property
    def name(self):
        return self.channel.channel_name

    @name.setter
    def name(self, val):
        self.channel.channel_name = '%s/avg' % val
        self.channel_min.channel_name = '%s/min' % val
        self.channel_max.channel_name = '%s/max' % val

        group_name = ''
        if '/' in val:
            group_name, _ = val.split('/')

        if self.chart is None:
            group_plot_title = self.plot_title
            if group_name != '':
                group_plot_title = '{}/{}'.format(group_name, self.plot_title)

            if group_plot_title not in Base.charts:
                Base.charts[group_plot_title] = mlogger.getEnv().create_chart([], group_plot_title)
            self.chart = Base.charts[group_plot_title]
            self.chart.chart_x_axis = self.chart_x_axis
            self.chart.chart_y_axis = self.chart_y_axis

        self.chart.bind_channel(self.channel)
        self.chart.bind_channel(self.channel_min)
        self.chart.bind_channel(self.channel_max)

    @property
    def value(self):
        return self._avg

    def reset(self):
        self._avg = None
        self._max_val = -np.inf
        self._min_val = np.inf
        self._total_weight = 0
        return self

    def get_video_memory(self):
        try:
            p = os.popen('nvidia-smi')
            content = p.read()
            p.close()
            if content == '':
                return 0.0

            gpu_percent_list = re.findall(r"[0-9]*0MiB", content)
            vv = []
            for g_v in gpu_percent_list:
                vv.append(float(g_v[:-3]))

            value = np.mean(np.array(vv))
            value = float(value)
            return value
        except:
            return 0.0

    def _update(self, *args, **kwargs):
        if self._avg is None:
            self._avg = 0.0

        # 获取内存占用率
        val = self.get_video_memory()

        # 更新
        r = self._total_weight / (1.0 + self._total_weight)
        self._avg = r * self._avg + (1 - r) * val
        self._total_weight += 1.0

        if val > self._max_val:
            self._max_val = val
        if val < self._min_val:
            self._min_val = val

        self.channel.update(self.time, self._avg)
        self.channel_min.update(self.time, self._min_val)
        self.channel_max.update(self.time, self._max_val)



# -*- coding: UTF-8 -*-
# @Time    : 2020-06-01 11:54
# @File    : httprpc.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
import requests
import json
import os
import tarfile
import sys
import urllib
import uuid
import gzip
import logging


class Resource(object):
    def __init__(self, rpc, resource):
        self._rpc = rpc
        self._resource = resource

    def slicing_upload(self, tag, **kwargs):
        # only support single file
        values = kwargs.values()
        if len(values) == 0:
            return

        file_path = list(kwargs.values())[0]
        if not os.path.exists(file_path):
            return

        if self._rpc.data is not None and type(self._rpc.data) == dict:
            kwargs.update(self._rpc.data)

        file_name = os.path.normpath(file_path).split('/')[-1]
        file_id = str(uuid.uuid4())
        file_size = os.path.getsize(file_path)
        chunk_size = 1024*1024 # 1M
        chunk_num = (file_size + chunk_size - 1)//chunk_size
        finished_size = 0
        api_url = '%s/upload/%s/' % (self._rpc.url, self._resource)
        try_times = 3
        fail_blocks = {}
        with open(file_path, 'rb') as fp:
            for chunk_id in range(chunk_num):
                remained_size = chunk_size
                if finished_size + chunk_size < file_size:
                    remained_size = chunk_size
                else:
                    remained_size = finished_size+chunk_size - file_size
                content = fp.read(remained_size)
                kwargs.update({'chunk_id': chunk_id, 'chunk_num': chunk_num, 'file_id':file_id, 'file_name': file_name, 'file_tag': tag})

                result = requests.post(api_url, kwargs, files={'file': content}, headers=self._rpc.headers)
                if result.status_code not in [200, 201]:
                    fail_blocks[chunk_id] = content
                    if len(fail_blocks) > 100:
                        logging.error('Always fail update, exit')
                        return

        if len(fail_blocks) > 0:
            for chunk_id, chuck_content in fail_blocks.items():
                result = requests.post(api_url, kwargs, files={'file': content}, headers=self._rpc.headers)
                if result.status_code not in [200, 201]:
                    logging.error('Always fail update, exit')
                    return

    def upload(self, *args, **kwargs):
        is_slicing = False
        if 'slicing' in kwargs:
            is_slicing = kwargs['slicing']
            kwargs.pop('slicing')
        tag = ''
        if 'tag' in kwargs:
            tag = kwargs['tag']
            kwargs.pop('tag')

        if is_slicing:
            # 开启分片上传
            response = self.slicing_upload(tag, **kwargs)
            return response

        files = {}
        for k, v in kwargs.items():
            if os.path.isdir(v):
                # compress
                normal_path = os.path.normpath(v)
                compressed_file_name = normal_path.split('/')[-1]
                compressed_file_path = os.path.join('/'.join(normal_path.split('/')[0:-1]), '%s.tar.gz' % compressed_file_name)

                if os.path.exists(compressed_file_path):
                    os.remove(compressed_file_path)

                tar = tarfile.open(compressed_file_path, 'w:gz')
                for f in os.listdir(v):
                    if f[0] == ".":
                        continue
                    tar.add(os.path.join(v, f), arcname=os.path.join(compressed_file_name, f))
                tar.close()
                
                v = compressed_file_path
            
            if not os.path.exists(v):
                continue
            
            if not k.startswith('FILE'):
                k = 'FILE_'+k
            files[k] = open(v, 'rb')
            
        if len(files) == 0:
            return {'status': 'ERROR', 'message': 'no files'}

        if self._rpc.data is not None and type(self._rpc.data) == dict:
            kwargs.update(self._rpc.data)
        
        api_url = '%s/upload/%s/' % (self._rpc.url, self._resource)

        result = requests.post(api_url, kwargs, files=files, headers=self._rpc.headers)
        if result.status_code not in [200, 201]:
            return {'status': 'ERROR'}

        response = json.loads(result.content)
        return response

    def __mkdir_p(self, dirname):
        """ make a dir recursively, but do nothing if the dir exists"""
        assert dirname is not None
        if dirname == '' or os.path.isdir(dirname):
            return
        try:
            os.makedirs(dirname)
        except OSError as e:
            if e.errno != 17:
                raise e

    # def __download(self, url, dir, fname=None):
    #     self.__mkdir_p(dir)
    #     if fname is None:
    #         fname = url.split('/')[-1]
    #     fpath = os.path.join(dir, fname)
    #
    #     def _progress(count, block_size, total_size):
    #         sys.stdout.write('\r>> downloading %s %.1f%%' %
    #                          (fname,
    #                           min(float(count * block_size) / total_size,
    #                               1.0) * 100.0))
    #         sys.stdout.flush()
    #
    #     try:
    #         fpath, _ = urllib.request.urlretrieve(url, fpath, reporthook=_progress)
    #         statinfo = os.stat(fpath)
    #         size = statinfo.st_size
    #     except:
    #         print("failed to download {}".format(url))
    #         return None
    #
    #     sys.stdout.write('\n')
    #     print('succesfully downloaded ' + fname + " " + str(size) + ' bytes.')
    #     return fpath

    def __get_file_name(self, headers):
        filename = ''
        if 'Content-Disposition' in headers and headers['Content-Disposition']:
            disposition_split = headers['Content-Disposition'].split(';')
            if len(disposition_split) > 1:
                if disposition_split[1].strip().lower().startswith('filename='):
                    filename = disposition_split[1].split('=')[-1]

        return filename

    def download(self, *args, **kwargs):
        # download and uncompress
        to_folder = kwargs.get('file_folder', "./")
        self.__mkdir_p(to_folder)

        download_file_name = ''
        if 'file_name' in kwargs:
            download_file_name = kwargs['file_name']

        if 'file_folder' in kwargs:
            kwargs.pop('file_folder')

        if self._rpc.data is not None and type(self._rpc.data) == dict:
            for k, v in self._rpc.data.items():
                if k not in kwargs:
                    kwargs.update({k: v})
        
        file_api_url = '%s/download/%s/' % (self._rpc.url, self._resource)
        is_success = True
        try:
            r = requests.get(file_api_url, kwargs, stream=True, headers=self._rpc.headers)
            file_name = self.__get_file_name(r.headers)
            if file_name != '' and download_file_name == '':
                download_file_name = file_name

            if download_file_name == '':
                download_file_name = str(uuid.uuid4())

            with open(os.path.join(to_folder, download_file_name), "wb") as pf:
                for chunk in r.iter_content(chunk_size=1024):
                    if chunk:
                        pf.write(chunk)

            logging.info('Successfully download {}'.format(download_file_name))
        except:
            logging.error('Failed to download {}'.format(download_file_name))
            if os.path.exists(os.path.join(to_folder, download_file_name)) and \
                    os.path.isfile(os.path.join(to_folder, download_file_name)):
                os.remove(os.path.join(to_folder, download_file_name))

            is_success = False

        if download_file_name.endswith('.tar.gz'):
            # uncompress
            tar = tarfile.open(os.path.join(to_folder, download_file_name))
            tar.extractall(to_folder)
            tar.close()
            logging.info('Successfully untar {}'.format(download_file_name))
    
        return {'status': 'SUCCESS'} if is_success else {'status': 'ERROR'}
    
    def __getattr__(self, action):
        if action not in ['get', 'post', 'put', 'patch', 'delete', 'upload', 'download']:
            return Resource(self._rpc, '%s/%s' % (self._resource, action))

        def func(*argc, **kwargs):
            api_url = '%s/%s/'%(self._rpc.url, self._resource)
            if self._rpc.data is not None and type(self._rpc.data) == dict:
                kwargs.update(self._rpc.data)

            try:
                if action == 'get':
                    result = requests.get(api_url, kwargs, headers=self._rpc.headers)
                    if result.status_code not in [200, 201]:
                        return {'status': 'ERROR'}
                    response = json.loads(result.content)
                    return response
                elif action == 'post':
                    result = requests.post(api_url, kwargs, headers=self._rpc.headers)
                    if result.status_code not in [200, 201]:
                        return {'status': 'ERROR'}
                    response = json.loads(result.content)
                    return response
                elif action == 'put':
                    result = requests.put(api_url, kwargs, headers=self._rpc.headers)
                    if result.status_code not in [200, 201]:
                        return {'status': 'ERROR'}
                    response = json.loads(result.content)
                    return response
                elif action == 'patch':
                    result = requests.patch(api_url, kwargs, headers=self._rpc.headers)
                    if result.status_code not in [200, 201]:
                        return {'status': 'ERROR'}
                    response = json.loads(result.content)
                    return response
                elif action == 'delete':
                    result = requests.delete(api_url, data=kwargs, headers=self._rpc.headers)
                    if result.status_code not in [200, 201]:
                        return {'status': 'ERROR'}
                    response = json.loads(result.content)
                    return response
                elif action == 'upload':
                    return None
                else:
                    return None
            except:
                return {'status': 'ERROR', 'message': 'connection error'}
            
        return func

        
class HttpRpc(object):
    def __init__(self, version, prefix, ip, port, token=None, **kwargs):
        self._ip = ip
        self._port = port
        self._prefix = prefix
        self._version = version
        self._token = token
        self._data = kwargs
    
    @property
    def version(self):
        return self._version
    
    @version.setter
    def version(self, val):
        self._version = val
    
    @property
    def ip(self):
        return self._ip
    
    @ip.setter
    def ip(self, val):
        self._ip = val
    
    @property
    def port(self):
        return self._port
    
    @port.setter
    def port(self,val):
        self._port = val
    
    @property
    def prefix(self):
        return self._prefix
    
    @prefix.setter
    def prefix(self, val):
        self._prefix = val
    
    @property
    def version(self):
        return self._version
    
    @version.setter
    def version(self, val):
        self._version = val
    
    @property
    def token(self):
        return self._token
    
    @token.setter
    def token(self, val):
        self._token = val
        
    @property    
    def url(self):
        request_url = 'http://%s:%d/%s' % (self.ip, self.port, self.prefix)
        return request_url
        
    @property
    def headers(self):
        if self.token is None:
            return None
        
        authorization = {'Authorization': "token " + self.token}
        return authorization
    
    @property
    def data(self):
        return self._data
    
    @data.setter
    def data(self, val):
        self._data = val
    
    def __getattr__(self, resource):
        return Resource(self, resource)



# -*- coding: UTF-8 -*-
# @Time    : 2020-06-25 23:28
# @File    : container.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function


class Container(object):
    def __init__(self):
        self._name = ''
        self._type = ''
        self._logger_obj = {}
    
    @property
    def name(self):
        return self._name

    @property
    def metrics(self):
        return [v for k,v in self._logger_obj.items() if v.logger_type == 'metric']

    def __setattr__(self, key, value):
        if key.startswith('_'):
            return super(Container, self).__setattr__(key, value)
        
        if type(value) == Container:
            if self.name != '':
                object.__setattr__(value, '_name', '{}/{}'.format(self.name, key))
            else:
                object.__setattr__(value, '_name', key)
        else:
            if self.name != '':
                value.name = '{}/{}'.format(self.name, key)
            else:
                value.name = key
                
        self.__dict__['_logger_obj'][key] = value
    
    def __getattr__(self, item):
        # if item not in self._logger_obj:
        #     return object.__getattribute__(self, item)
        if item not in self._logger_obj:
            return self.__dict__[item]

        return self.__dict__['_logger_obj'][item]



from setuptools import setup, find_packages

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='antvis',
      version='0.1.7',
      description='machine learning manage client',
      __short_description__='machine learning manage client',
      url='http://www.mltalker.com',
      author='jian',
      author_email='jian@mltalker.com',
      packages=find_packages(),
      long_description=readme(),
      include_package_data=True,
      zip_safe=False,)



# -*- coding: UTF-8 -*-
# @Time    : 2020-06-30 13:37
# @File    : __init__.py.py
# @Author  : jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function

from antvis.client.mlogger.metric.base import *
from antvis.client.dashboard import *
from antvis.client import mlogger
import time
import logging


class Image(Base):
    def __init__(self, plot_title, **kwargs):
        super(Image, self).__init__(plot_title, 'complex', **kwargs)
        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='IMAGE', **self.channel_config)
        
    @property
    def value(self):
        return self._val
    
    def _update(self, val):
        if type(val) == list:
            val = np.array(val)
        
        if type(val) != np.ndarray:
            logging.error('Image logger dont support non numpy.ndarray data')
            raise NotImplementedError
        
        assert(len(val.shape) == 2 or len(val.shape) == 3)
        if len(val.shape) == 3:
            assert(val.shape[2] == 3)
        
        self._val = val
        self.channel.update(self.time, self.value)

    def _get(self, data):
        return data


class Histogram(Base):
    def __init__(self, plot_title, **kwargs):
        super(Histogram, self).__init__(plot_title, 'complex', **kwargs)
        
        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='HISTOGRAM', **self.channel_config)
        
    @property
    def value(self):
        return self._val
    
    def _update(self, val):
        self._val = val
        self.channel.update(self.time, self.value)


class Bar(Base):
    def __init__(self, plot_title, **kwargs):
        super(Bar, self).__init__(plot_title, 'complex', **kwargs)

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='BAR',
                                                       **self.channel_config)

    @property
    def value(self):
        return self._val

    def _update(self, x,y):
        self._val = (x,y)
        self.channel.update(x, y)


class Heatmap(Base):
    def __init__(self, plot_title, **kwargs):
        super(Heatmap, self).__init__(plot_title, 'complex', **kwargs)
        
        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='HEATMAP', **self.channel_config)

    @property
    def value(self):
        return self._val
    
    def _update(self, val):
        if type(val) != np.ndarray:
            val = np.array(val)
        
        assert(len(val.shape) == 2)
        if self.chart.chart_x_axis == '':
            self.chart.chart_x_axis = ','.join([str(i) for i in range(val.shape[0])])
            
        if self.chart.chart_y_axis == '':
            self.chart.chart_y_axis = ','.join([str(i) for i in range(val.shape[1])])
        
        val = val.tolist()
        self._val = val
        self.channel.update(self.time, self.value)
        
        
class Svg(Base):
    def __init__(self, plot_title, **kwargs):
        super(Svg, self).__init__(plot_title, 'complex', **kwargs)

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='SVG', **self.channel_config)

    @property
    def value(self):
        return self._val

    def _update(self, val):
        assert(type(val) == str)
        self._val = val
        self.channel.update(self.time, self.value)


class Text(Base):
    def __init__(self, plot_title, **kwargs):
        super(Text, self).__init__(plot_title, 'complex', **kwargs)
        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='TEXT', **self.channel_config)

    @property
    def value(self):
        return self._val

    def _update(self, val):
        if type(val) != str:
            val = str(val)
            
        self._val = val
        self.channel.update(self.time, self.value)

    def _timestamp(self):
        time_str = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
        return time_str

    def info(self, val):
        self.update("{} [INFO]\t{}".format(self._timestamp(), val))

    def debug(self, val):
        self.update("{} [DEBUG]\t{}".format(self._timestamp(), val))

    def warn(self, val):
        self.update("{} [WARN]\t{}".format(self._timestamp(), val))


class Table(Base):
    def __init__(self, plot_title, **kwargs):
        super(Table, self).__init__(plot_title, 'complex', **kwargs)

        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='TABLE', **self.channel_config)

    @property
    def value(self):
        return self._val

    def _update(self, *args, **kwargs):
        self.channel.update(self.time, self.value)
    
    def __setattr__(self, key, value):
        if not key.startswith('r'):
            return super(Table, self).__setattr__(key, value)
        
        if len(key) != 4:
            return
        r,i,c,j = key
        if r!='r' or c!='c':
            return
        
        i = int(i)
        j = int(j)
        if i < 0 or j < 0:
            raise NotImplementedError
        
        if self._val is None or len(self._val) == 0:
            self._val = [['' for jj in range(j+1)] for ii in range(i+1)]
        
        if len(self._val) < (i+1):
            for _ in range((i+1) - len(self._val)):
                self._val.append(['' for jj in range(len(self._val[0]))])
        
        if len(self._val[0]) < (j+1):
            for r_index in range(len(self._val)):
                self._val[r_index].extend(['' for jj in range((j+1) - len(self._val[r_index]))])
        
        self._val[i][j] = value


class Scatter(Base):
    def __init__(self, plot_title, **kwargs):
        super(Scatter, self).__init__(plot_title, 'complex', **kwargs)
        self.channel = mlogger.getEnv().create_channel(str(uuid.uuid4()), channel_type='SCATTER', time_series=False, **self.channel_config)

    @property
    def value(self):
        return self._val

    def _update(self, val):
        # val format : [[],[]]
        if type(val) == list:
            val = np.array(val)

        if type(val) != np.ndarray:
            logging.error('Image logger dont support non numpy.ndarray data')
            raise NotImplementedError

        assert (len(val.shape) == 2)
        assert (val.shape[0] == 2 or val.shape[1] == 2)
        if val.shape[0] == 2:
            val = val.tolist()
        else:
            val = np.transpose(val)
            val = val.tolist()

        self._val = val
        x, y = val
        self.channel.update(x, y)
